@@ -26,14 +26,12 @@
 
 #include "gnome-settings-plugin.h"
 #include "gnome-settings-profile.h"
+#include "gnome-settings-bus.h"
 #include "gsd-smartcard-manager.h"
 #include "gsd-smartcard-service.h"
 #include "gsd-smartcard-enum-types.h"
 #include "gsd-smartcard-utils.h"
 
-#include "org.gnome.ScreenSaver.h"
-#include "org.gnome.SessionManager.h"
-
 #include <prerror.h>
 #include <prinit.h>
 #include <nss.h>
@@ -728,32 +726,6 @@ gsd_smartcard_manager_stop (GsdSmartcardManager *self)
         g_clear_object (&priv->screen_saver);
 }
 
-static void
-on_got_screen_saver_to_lock_screen (GObject             *object,
-                                    GAsyncResult        *result,
-                                    GsdSmartcardManager *self)
-{
-        GsdSmartcardManagerPrivate *priv = self->priv;
-        GsdScreenSaver *screen_saver;
-        GError *error = NULL;
-
-        screen_saver = gsd_screen_saver_proxy_new_for_bus_finish (result, &error);
-
-        if (screen_saver == NULL) {
-                g_warning ("Couldn't find screen saver service to lock screen: %s",
-                           error->message);
-                g_error_free (error);
-                return;
-        }
-
-        if (priv->screen_saver != NULL)
-                g_object_unref (screen_saver);
-
-        priv->screen_saver = screen_saver;
-
-        lock_screen (self);
-}
-
 static void
 on_screen_locked (GsdScreenSaver      *screen_saver,
                   GAsyncResult        *result,
@@ -776,16 +748,8 @@ lock_screen (GsdSmartcardManager *self)
 {
         GsdSmartcardManagerPrivate *priv = self->priv;
 
-        if (priv->screen_saver == NULL) {
-                gsd_screen_saver_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-                                                    G_DBUS_PROXY_FLAGS_NONE,
-                                                    "org.gnome.ScreenSaver",
-                                                    "/org/gnome/ScreenSaver",
-                                                    priv->cancellable,
-                                                    (GAsyncReadyCallback) on_got_screen_saver_to_lock_screen,
-                                                    self);
-                return;
-        }
+        if (priv->screen_saver == NULL)
+                priv->screen_saver = gnome_settings_bus_get_screen_saver_proxy ();
 
         gsd_screen_saver_call_lock (priv->screen_saver,
                                     priv->cancellable,
@@ -793,32 +757,6 @@ lock_screen (GsdSmartcardManager *self)
                                     self);
 }
 
-static void
-on_got_session_manager_to_log_out (GObject             *object,
-                                   GAsyncResult        *result,
-                                   GsdSmartcardManager *self)
-{
-        GsdSmartcardManagerPrivate *priv = self->priv;
-        GsdSessionManager *session_manager;
-        GError *error = NULL;
-
-        session_manager = gsd_session_manager_proxy_new_for_bus_finish (result, &error);
-
-        if (session_manager == NULL) {
-                g_warning ("Couldn't find session manager service to log out: %s",
-                           error->message);
-                g_error_free (error);
-                return;
-        }
-
-        if (priv->session_manager != NULL)
-                g_object_unref (session_manager);
-
-        priv->session_manager = session_manager;
-
-        log_out (self);
-}
-
 static void
 on_logged_out (GsdSessionManager   *session_manager,
                GAsyncResult        *result,
@@ -841,16 +779,8 @@ log_out (GsdSmartcardManager *self)
 {
         GsdSmartcardManagerPrivate *priv = self->priv;
 
-        if (priv->session_manager == NULL) {
-                gsd_session_manager_proxy_new_for_bus (G_BUS_TYPE_SESSION,
-                                                       G_DBUS_PROXY_FLAGS_NONE,
-                                                       "org.gnome.SessionManager",
-                                                       "/org/gnome/SessionManager",
-                                                       priv->cancellable,
-                                                       (GAsyncReadyCallback) on_got_session_manager_to_log_out,
-                                                       self);
-                return;
-        }
+        if (priv->session_manager == NULL)
+                priv->session_manager = gnome_settings_bus_get_session_proxy ();
 
         gsd_session_manager_call_logout (priv->session_manager,
                                          GSD_SESSION_MANAGER_LOGOUT_MODE_FORCE,
