@@ -30,6 +30,7 @@
 
 #include "gsd-wacom-osd-window.h"
 #include "gsd-wacom-device.h"
+#include "gsd-wacom-button-editor.h"
 #include "gsd-enums.h"
 
 #define ROTATION_KEY                "rotation"
@@ -39,13 +40,17 @@
 #define RES_PATH                    "/org/gnome/settings-daemon/plugins/wacom/"
 
 #define BACK_OPACITY		0.8
+#define OPACITY_IN_EDITION	"0.5"
 #define INACTIVE_COLOR		"#ededed"
 #define ACTIVE_COLOR		"#729fcf"
 #define STROKE_COLOR		"#000000"
 #define DARK_COLOR		"#535353"
 #define BACK_COLOR		"#000000"
 
-#define ELEVATOR_TIMEOUT	250 /* ms */
+#define BUTTON_HIGHLIGHT_DURATION     250 /* ms */
+#define BUTTON_TRANSITION_DURATION    150 /* ms */
+#define BUTTON_TIMER_STEP             25  /* ms */
+#define BUTTON_FULL_TRANSITION        1.0 /* % */
 
 #define CURSOR_HIDE_TIMEOUT	2 /* seconds */
 
@@ -224,8 +229,7 @@ enum {
 	PROP_OSD_BUTTON_CLASS,
 	PROP_OSD_BUTTON_LABEL,
 	PROP_OSD_BUTTON_ACTIVE,
-	PROP_OSD_BUTTON_VISIBLE,
-	PROP_OSD_BUTTON_AUTO_OFF
+	PROP_OSD_BUTTON_VISIBLE
 };
 
 #define GSD_WACOM_OSD_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
@@ -244,8 +248,14 @@ struct GsdWacomOSDButtonPrivate {
 	GsdWacomTabletButtonPos   position;
 	gboolean                  active;
 	gboolean                  visible;
-	guint                     auto_off;
-	guint                     timeout;
+	GdkRGBA                   active_color;
+	GdkRGBA                   inactive_color;
+
+	/* Animations */
+	gboolean                  next_state;            /* what the next state should be, "active" or not */
+	guint                     timeout_id;            /* ms */
+	gint                      elapsed_time;          /* ms */
+	gdouble                   transition_percentage; /* 0.0 to 1.0 */
 };
 
 static void     gsd_wacom_osd_button_class_init  (GsdWacomOSDButtonClass *klass);
@@ -321,15 +331,6 @@ gsd_wacom_osd_button_set_location (GsdWacomOSDButton        *osd_button,
 	osd_button->priv->label_y = y;
 }
 
-static void
-gsd_wacom_osd_button_set_auto_off (GsdWacomOSDButton        *osd_button,
-				   guint                     timeout)
-{
-	g_return_if_fail (GSD_IS_WACOM_OSD_BUTTON (osd_button));
-
-	osd_button->priv->auto_off = timeout;
-}
-
 static void
 gsd_wacom_osd_button_redraw (GsdWacomOSDButton *osd_button)
 {
@@ -344,11 +345,51 @@ gsd_wacom_osd_button_redraw (GsdWacomOSDButton *osd_button)
 static gboolean
 gsd_wacom_osd_button_timer (GsdWacomOSDButton *osd_button)
 {
-	/* Auto de-activate the button */
-	osd_button->priv->active = FALSE;
+	gint     total_timeout;
+	gdouble  transition_step;
+	gboolean ret = G_SOURCE_CONTINUE;
+
+	total_timeout = BUTTON_TRANSITION_DURATION;
+	/* if the state is active, then we need it to be "on" at least
+	 * for the time of BUTTON_HIGHLIGHT_DURATION after the fade-in */
+	if (osd_button->priv->active)
+		total_timeout += BUTTON_HIGHLIGHT_DURATION;
+
+	transition_step = (gdouble) BUTTON_TIMER_STEP / BUTTON_TRANSITION_DURATION;
+	osd_button->priv->transition_percentage = MIN (osd_button->priv->transition_percentage + transition_step,
+						       BUTTON_FULL_TRANSITION);
+
+	osd_button->priv->elapsed_time += BUTTON_TIMER_STEP;
+	if (osd_button->priv->elapsed_time > total_timeout) {
+		/* If the next state and the current one are the same, then
+		 * we finish this timer */
+		if (osd_button->priv->active == osd_button->priv->next_state) {
+			ret = G_SOURCE_REMOVE;
+			osd_button->priv->timeout_id = 0;
+		} else
+			osd_button->priv->active = osd_button->priv->next_state;
+
+		osd_button->priv->elapsed_time = 0;
+		osd_button->priv->transition_percentage = 0;
+	}
+
 	gsd_wacom_osd_button_redraw (osd_button);
 
-	return FALSE;
+	return ret;
+}
+
+static void
+invert_button_color_transition (GsdWacomOSDButton *osd_button)
+{
+	osd_button->priv->elapsed_time = (BUTTON_HIGHLIGHT_DURATION + BUTTON_TRANSITION_DURATION) - osd_button->priv->elapsed_time;
+	osd_button->priv->transition_percentage = BUTTON_FULL_TRANSITION - osd_button->priv->transition_percentage;
+}
+
+static void
+extend_on_state (GsdWacomOSDButton *osd_button)
+{
+	osd_button->priv->elapsed_time = BUTTON_TRANSITION_DURATION;
+	osd_button->priv->transition_percentage = BUTTON_FULL_TRANSITION;
 }
 
 static void
@@ -359,24 +400,26 @@ gsd_wacom_osd_button_set_active (GsdWacomOSDButton *osd_button,
 
 	g_return_if_fail (GSD_IS_WACOM_OSD_BUTTON (osd_button));
 
+        osd_button->priv->next_state = active;
 	previous_state = osd_button->priv->active;
-	if (osd_button->priv->auto_off > 0) {
-		/* For auto-off buttons, apply only if active, de-activation is done in the timeout */
-		if (active == TRUE)
-			osd_button->priv->active = active;
-
-		if (osd_button->priv->timeout)
-			g_source_remove (osd_button->priv->timeout);
-		osd_button->priv->timeout = g_timeout_add (osd_button->priv->auto_off,
-		                                           (GSourceFunc) gsd_wacom_osd_button_timer,
-		                                           osd_button);
-	} else {
-		/* Whereas for other buttons, apply the change straight away */
+	if (osd_button->priv->timeout_id == 0) {
 		osd_button->priv->active = active;
+		osd_button->priv->timeout_id = g_timeout_add (BUTTON_TIMER_STEP,
+							      (GSourceFunc) gsd_wacom_osd_button_timer,
+							      osd_button);
+	} else if (osd_button->priv->next_state) {
+		/* it was on and now should be on again */
+		if (previous_state == active) {
+			/* if the button has transitioned to on already, then it
+			 * needs to remain on for the default amount of time again */
+			if (osd_button->priv->elapsed_time > BUTTON_TRANSITION_DURATION)
+				extend_on_state (osd_button);
+		} else {
+			/* if we have an on-going timeout which was fading out and
+			 * now we need to fade it in again, we just invert the transition*/
+			invert_button_color_transition (osd_button);
+		}
 	}
-
-	if (previous_state != osd_button->priv->active)
-		gsd_wacom_osd_button_redraw (osd_button);
 }
 
 static void
@@ -428,9 +471,6 @@ gsd_wacom_osd_button_set_property (GObject        *object,
 	case PROP_OSD_BUTTON_VISIBLE:
 		gsd_wacom_osd_button_set_visible (osd_button, g_value_get_boolean (value));
 		break;
-	case PROP_OSD_BUTTON_AUTO_OFF:
-		gsd_wacom_osd_button_set_auto_off (osd_button, g_value_get_uint (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -463,9 +503,6 @@ gsd_wacom_osd_button_get_property (GObject        *object,
 	case PROP_OSD_BUTTON_VISIBLE:
 		g_value_set_boolean (value, osd_button->priv->visible);
 		break;
-	case PROP_OSD_BUTTON_AUTO_OFF:
-		g_value_set_uint (value, osd_button->priv->auto_off);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -516,15 +553,6 @@ gsd_wacom_osd_button_class_init (GsdWacomOSDButtonClass *klass)
 	                                                       "Whether the button is visible",
 	                                                       TRUE,
 	                                                       G_PARAM_READWRITE));
-	g_object_class_install_property (object_class,
-	                                 PROP_OSD_BUTTON_AUTO_OFF,
-	                                 g_param_spec_uint    ("auto-off",
-	                                                       "Auto Off",
-	                                                       "Timeout before button disables itself automatically",
-	                                                       0,
-	                                                       G_MAXUINT,
-	                                                       0, /* disabled by default */
-	                                                       G_PARAM_READWRITE));
 
 	g_type_class_add_private (klass, sizeof (GsdWacomOSDButtonPrivate));
 }
@@ -533,6 +561,9 @@ static void
 gsd_wacom_osd_button_init (GsdWacomOSDButton *osd_button)
 {
 	osd_button->priv = GSD_WACOM_OSD_BUTTON_GET_PRIVATE (osd_button);
+
+	gdk_rgba_parse (&osd_button->priv->active_color, ACTIVE_COLOR);
+	gdk_rgba_parse (&osd_button->priv->inactive_color, INACTIVE_COLOR);
 }
 
 static void
@@ -550,8 +581,8 @@ gsd_wacom_osd_button_finalize (GObject *object)
 
 	priv = osd_button->priv;
 
-	if (priv->timeout > 0)
-		g_source_remove (priv->timeout);
+	if (priv->timeout_id > 0)
+		g_source_remove (priv->timeout_id);
 	g_clear_pointer (&priv->id, g_free);
 	g_clear_pointer (&priv->class, g_free);
 	g_clear_pointer (&priv->label, g_free);
@@ -588,6 +619,67 @@ get_actual_position (GsdWacomTabletButtonPos position,
 	return position;
 }
 
+static gchar *
+rgb_color_to_hex (GdkRGBA *color)
+{
+	gchar *hex_color;
+
+	hex_color = g_strdup_printf ("#%02X%02X%02X",
+				     (guint) (color->red * 255),
+				     (guint) (color->green * 255),
+				     (guint) (color->blue * 255));
+
+	return hex_color;
+}
+
+static GdkRGBA *
+transition_rgba_colors (GdkRGBA *from_rgba,
+			GdkRGBA *to_rgba,
+			gdouble  transition_percentage)
+{
+	GdkRGBA *new_color = gdk_rgba_copy (from_rgba);
+
+	if (transition_percentage == 0.0)
+		return new_color;
+
+	new_color->red -= (from_rgba->red - to_rgba->red) * transition_percentage;
+	new_color->green -= (from_rgba->green - to_rgba->green) * transition_percentage;
+	new_color->blue -= (from_rgba->blue - to_rgba->blue) * transition_percentage;
+
+	return new_color;
+}
+
+static gchar *
+gsd_wacom_osd_button_get_color_str (GsdWacomOSDButton *osd_button)
+{
+	GdkRGBA *current_color, *from_color, *to_color;
+	gboolean free_current_color = FALSE;
+	gchar   *color_str;
+
+	/* If we got an on-going transition, then we need to apply it */
+	if (osd_button->priv->timeout_id > 0) {
+		if (osd_button->priv->active) {
+			from_color = &osd_button->priv->inactive_color;
+			to_color = &osd_button->priv->active_color;
+		} else {
+			from_color = &osd_button->priv->active_color;
+			to_color = &osd_button->priv->inactive_color;
+		}
+
+		current_color = transition_rgba_colors (from_color, to_color, osd_button->priv->transition_percentage);
+
+		free_current_color = TRUE;
+	} else
+		current_color = osd_button->priv->active ? &osd_button->priv->active_color : &osd_button->priv->inactive_color;
+
+	color_str = rgb_color_to_hex (current_color);
+
+	if (free_current_color)
+		gdk_rgba_free (current_color);
+
+	return color_str;
+}
+
 static void
 gsd_wacom_osd_button_draw_label (GsdWacomOSDButton *osd_button,
 			         GtkStyleContext   *style_context,
@@ -600,7 +692,7 @@ gsd_wacom_osd_button_draw_label (GsdWacomOSDButton *osd_button,
 	PangoRectangle            logical_rect;
 	GsdWacomTabletButtonPos   actual_position;
 	double                    lx, ly;
-	gchar                    *markup;
+	gchar                    *markup, *color_str;
 
 	g_return_if_fail (GSD_IS_WACOM_OSD_BUTTON (osd_button));
 
@@ -610,10 +702,9 @@ gsd_wacom_osd_button_draw_label (GsdWacomOSDButton *osd_button,
 
 	actual_position = get_actual_position (priv->position, rotation);
 	layout = pango_layout_new (pango_context);
-	if (priv->active)
-		markup = g_strdup_printf ("<span foreground=\"" ACTIVE_COLOR "\" weight=\"normal\">%s</span>", priv->label);
-	else
-		markup = g_strdup_printf ("<span foreground=\"" INACTIVE_COLOR "\" weight=\"normal\">%s</span>", priv->label);
+	color_str = gsd_wacom_osd_button_get_color_str (osd_button);
+	markup = g_strdup_printf ("<span foreground=\"%s\" weight=\"normal\">%s</span>", color_str, priv->label);
+	g_free (color_str);
 	pango_layout_set_markup (layout, markup, -1);
 	g_free (markup);
 
@@ -653,7 +744,8 @@ gsd_wacom_osd_button_draw_label (GsdWacomOSDButton *osd_button,
 enum {
   PROP_OSD_WINDOW_0,
   PROP_OSD_WINDOW_MESSAGE,
-  PROP_OSD_WINDOW_GSD_WACOM_DEVICE
+  PROP_OSD_WINDOW_GSD_WACOM_DEVICE,
+  PROP_OSD_WINDOW_EDITION_MODE
 };
 
 #define GSD_WACOM_OSD_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), \
@@ -669,8 +761,14 @@ struct GsdWacomOSDWindowPrivate
 	GdkRectangle              monitor_area;
 	GdkRectangle              tablet_area;
 	char                     *message;
+	char                     *edition_mode_message;
+	char                     *regular_mode_message;
 	GList                    *buttons;
 	guint                     cursor_timeout;
+	gboolean                  edition_mode;
+	GsdWacomOSDButton        *current_button;
+	GtkWidget                *editor;
+	GtkWidget                *change_mode_button;
 };
 
 static void     gsd_wacom_osd_window_class_init  (GsdWacomOSDWindowClass *klass);
@@ -679,6 +777,12 @@ static void     gsd_wacom_osd_window_finalize    (GObject                *object
 
 G_DEFINE_TYPE (GsdWacomOSDWindow, gsd_wacom_osd_window, GTK_TYPE_WINDOW)
 
+static gboolean
+osd_window_editing_button (GsdWacomOSDWindow *self)
+{
+	return self->priv->edition_mode && gtk_widget_get_visible (self->priv->editor);
+}
+
 static void
 gsd_wacom_osd_window_update (GsdWacomOSDWindow *osd_window)
 {
@@ -715,18 +819,35 @@ gsd_wacom_osd_window_update (GsdWacomOSDWindow *osd_window)
 	/* Build the buttons section */
 	buttons_section = g_strdup ("");
 	for (l = osd_window->priv->buttons; l != NULL; l = l->next) {
+		gchar *color_str;
+		const gchar *css;
 		GsdWacomOSDButton *osd_button = l->data;
 
 		if (osd_button->priv->visible == FALSE)
 			continue;
 
-		if (osd_button->priv->active) {
-			buttons_section = g_strconcat (buttons_section,
-			                               ".", osd_button->priv->class, " {\n"
-			                               "      stroke:   active_color !important;\n"
-			                               "      fill:     active_color !important;\n"
-			                               "    }\n",
-			                               NULL);
+		if (osd_window_editing_button (osd_window) &&
+		    osd_button != osd_window->priv->current_button) {
+			css = "%s.%s {\n"
+				"      opacity: %s\n"
+				"}\n";
+			buttons_section = g_strdup_printf (css,
+							   buttons_section,
+							   osd_button->priv->class,
+							   OPACITY_IN_EDITION,
+							   NULL);
+		} else {
+			color_str = gsd_wacom_osd_button_get_color_str (osd_button);
+			css = "%s.%s {\n"
+				"      stroke: %s !important;\n"
+				"      fill: %s !important;\n"
+				"}\n";
+			buttons_section = g_strdup_printf (css,
+							   buttons_section,
+							   osd_button->priv->class,
+							   color_str,
+							   color_str);
+			g_free (color_str);
 		}
 	}
 	replace_string (&css_string, "buttons_section", buttons_section);
@@ -767,7 +888,7 @@ gsd_wacom_osd_window_draw_message (GsdWacomOSDWindow   *osd_window,
 	double x;
 	double y;
 
-	if (osd_window->priv->message == NULL)
+	if (osd_window->priv->message == NULL || osd_window_editing_button (osd_window))
 		return;
 
 	layout = pango_layout_new (pango_context);
@@ -1115,10 +1236,18 @@ get_elevator_current_mode (GsdWacomOSDWindow    *osd_window,
 	return mode;
 }
 
+static void
+redraw_window (GsdWacomOSDWindow *self)
+{
+	GdkWindow *window;
+
+	window = gtk_widget_get_window (GTK_WIDGET (self));
+	gdk_window_invalidate_rect (window, NULL, FALSE);
+}
+
 static GsdWacomOSDButton *
 gsd_wacom_osd_window_add_button_with_dir (GsdWacomOSDWindow    *osd_window,
                                           GsdWacomTabletButton *tablet_button,
-                                          guint                 timeout,
                                           GtkDirectionType      dir)
 {
 	GsdWacomOSDButton    *osd_button;
@@ -1138,7 +1267,6 @@ gsd_wacom_osd_window_add_button_with_dir (GsdWacomOSDWindow    *osd_window,
 
 	gsd_wacom_osd_button_set_button_type (osd_button, tablet_button->type);
 	gsd_wacom_osd_button_set_position (osd_button, tablet_button->pos);
-	gsd_wacom_osd_button_set_auto_off (osd_button, timeout);
 	osd_window->priv->buttons = g_list_append (osd_window->priv->buttons, osd_button);
 
 	return osd_button;
@@ -1156,7 +1284,6 @@ gsd_wacom_osd_window_add_tablet_button (GsdWacomOSDWindow    *osd_window,
 	case WACOM_TABLET_BUTTON_TYPE_HARDCODED:
 		osd_button = gsd_wacom_osd_window_add_button_with_dir (osd_window,
 		                                                       tablet_button,
-		                                                       0,
 		                                                       0);
 		gsd_wacom_osd_button_set_visible (osd_button, TRUE);
 		break;
@@ -1167,14 +1294,12 @@ gsd_wacom_osd_window_add_tablet_button (GsdWacomOSDWindow    *osd_window,
 		/* Add 2 buttons per elevator, one "Up"... */
 		osd_button = gsd_wacom_osd_window_add_button_with_dir (osd_window,
 		                                                       tablet_button,
-		                                                       ELEVATOR_TIMEOUT,
 		                                                       GTK_DIR_UP);
 		gsd_wacom_osd_button_set_visible (osd_button, tablet_button->idx == mode);
 
 		/* ... and one "Down" */
 		osd_button = gsd_wacom_osd_window_add_button_with_dir (osd_window,
 		                                                       tablet_button,
-		                                                       ELEVATOR_TIMEOUT,
 		                                                       GTK_DIR_DOWN);
 		gsd_wacom_osd_button_set_visible (osd_button, tablet_button->idx == mode);
 
@@ -1185,48 +1310,57 @@ gsd_wacom_osd_window_add_tablet_button (GsdWacomOSDWindow    *osd_window,
 	}
 }
 
-static gboolean
-call_device_configuration (GsdWacomDevice *device)
+static char *
+get_regular_mode_message (GsdWacomOSDWindow *osd_window)
 {
-	gboolean success;
-	gchar *command;
-	const gchar *device_name;
-	GError *error = NULL;
+	const gchar *name;
+	gchar       *message;
 
-	device_name = gsd_wacom_device_get_name (device);
-	command = g_strdup_printf ("gnome-control-center wacom \"%s\"", device_name);
-	success = g_spawn_command_line_async (command, &error);
-	if (!success) {
-		g_warning ("Failure launching gnome-control-center: %s\n%s",
-			   error->message, command);
-		g_error_free (error);
-	}
-	g_free (command);
+	name = gsd_wacom_device_get_name (osd_window->priv->pad);
+	message = g_strdup_printf ("<big><b>%s</b></big>\n<span foreground=\"%s\">%s</span>",
+				   name, INACTIVE_COLOR, _("(press any key to exit)"));
 
-	return success;
+	return message;
+}
+
+static char *
+get_edition_mode_message (GsdWacomOSDWindow *osd_window)
+{
+	return g_strdup_printf ("<big><b>%s</b></big>\n<span foreground=\"%s\">%s</span>",
+				_("Push a button to configure"), INACTIVE_COLOR, _("(Esc to cancel)"));
 }
 
 static void
-on_configure_button_clicked (GtkButton         *button,
-			     GsdWacomOSDWindow *window)
+close_editor (GsdWacomOSDWindow *self)
 {
-	if (call_device_configuration (window->priv->pad))
-		gtk_widget_destroy (GTK_WIDGET (window));
+	if (self->priv->current_button)
+		self->priv->current_button->priv->visible = TRUE;
+
+	gtk_widget_hide (self->priv->editor);
+	self->priv->current_button = NULL;
 }
 
-static GtkWidget *
-create_osd_configure_button (GsdWacomOSDWindow *window)
+static void
+edition_mode_changed (GsdWacomOSDWindow *self)
 {
-	GtkWidget *configure_button;
-	GtkStyleContext *style_context;
+	if (self->priv->edition_mode)
+		gsd_wacom_osd_window_set_message (self, self->priv->edition_mode_message);
+	else {
+		gsd_wacom_osd_window_set_message (self, self->priv->regular_mode_message);
 
-	configure_button = gtk_button_new_with_label (_("Configure device"));
-	g_object_set (configure_button, "halign", GTK_ALIGN_CENTER, NULL);
+		close_editor (self);
+	}
 
-	style_context = gtk_widget_get_style_context (configure_button);
-	gtk_style_context_add_class (style_context, "osd");
+	redraw_window (self);
+}
 
-	return configure_button;
+static void
+on_change_mode_button_clicked (GtkButton         *button,
+			       GsdWacomOSDWindow *window)
+{
+	window->priv->edition_mode = !window->priv->edition_mode;
+
+	edition_mode_changed (window);
 }
 
 static void
@@ -1332,22 +1466,111 @@ display_relative_rotation (GsdWacomRotation device_rotation,
 }
 
 static void
-gsd_wacom_osd_window_mapped (GtkWidget *widget,
-                             gpointer   data)
+grab_keyboard (GsdWacomOSDWindow *self)
 {
-	GsdWacomOSDWindow *osd_window = GSD_WACOM_OSD_WINDOW (widget);
+	GdkDevice        *kbd = NULL;
+	GdkWindow        *gdk_window;
+	GdkDisplay       *display;
+	GdkDeviceManager *device_manager;
+	GList            *devices, *l;
 
-	g_return_if_fail (GSD_IS_WACOM_OSD_WINDOW (osd_window));
+	gdk_window = gtk_widget_get_window (GTK_WIDGET (self));
+	display = gtk_widget_get_display (GTK_WIDGET (self));
+	device_manager = gdk_display_get_device_manager (display);
+	devices = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
 
-	/* Position the window at its expected postion before moving
-	 * to fullscreen, so the window will be on the right monitor.
-	 */
-	gtk_window_move (GTK_WINDOW (osd_window),
-	                 osd_window->priv->screen_area.x,
-	                 osd_window->priv->screen_area.y);
+	for (l = devices; l != NULL; l = l->next) {
+		GdkDevice *current_device;
 
-	gtk_window_fullscreen (GTK_WINDOW (osd_window));
-	gtk_window_set_keep_above (GTK_WINDOW (osd_window), TRUE);
+		current_device = l->data;
+		if (gdk_device_get_source (current_device) == GDK_SOURCE_KEYBOARD) {
+			kbd = current_device;
+			break;
+		}
+	}
+	g_list_free (devices);
+
+	g_assert (kbd);
+
+	gdk_device_grab (kbd, gdk_window, GDK_OWNERSHIP_WINDOW, FALSE,
+			 GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
+			 NULL, GDK_CURRENT_TIME);
+}
+
+static void
+gsd_wacom_osd_window_show (GtkWidget *widget)
+{
+	GTK_WIDGET_CLASS (gsd_wacom_osd_window_parent_class)->show (widget);
+
+	grab_keyboard (GSD_WACOM_OSD_WINDOW (widget));
+}
+
+static void
+on_button_edited (GsdWacomButtonEditor *editor,
+		  GsdWacomOSDWindow    *self)
+{
+	GsdWacomTabletButton *button;
+	GtkDirectionType      dir;
+	char                 *str;
+
+	button = gsd_wacom_button_editor_get_button (editor, &dir);
+
+	if (button == NULL || self->priv->current_button == NULL)
+		return;
+
+	str = get_tablet_button_label (self->priv->pad, button, dir);
+	gsd_wacom_osd_button_set_label (self->priv->current_button, str);
+	g_free (str);
+
+	gsd_wacom_osd_button_redraw (self->priv->current_button);
+}
+
+static void
+on_button_editing_done (GtkWidget         *editor,
+			GsdWacomOSDWindow *self)
+{
+	close_editor (self);
+	redraw_window (self);
+	grab_keyboard (self);
+}
+
+static gboolean
+on_get_child_position (GtkOverlay        *overlay,
+		       GtkWidget         *widget,
+		       GdkRectangle      *allocation,
+		       GsdWacomOSDWindow *self)
+{
+	GsdWacomOSDButton       *button;
+	GtkRequisition           requisition;
+	GsdWacomTabletButtonPos  position;
+
+	button = self->priv->current_button;
+
+	if (button == NULL)
+		return FALSE;
+
+	gtk_widget_get_preferred_size (widget, NULL, &requisition);
+
+	allocation->x = button->priv->label_x;
+	allocation->y = button->priv->label_y;
+	allocation->width = requisition.width;
+	allocation->height = requisition.height;
+
+	position = get_actual_position (button->priv->position, self->priv->rotation);
+
+	if (position == WACOM_TABLET_BUTTON_POS_LEFT) {
+		allocation->y -= requisition.height / 2.0;
+	} else if (position == WACOM_TABLET_BUTTON_POS_RIGHT) {
+		allocation->x -= requisition.width;
+		allocation->y -= requisition.height / 2.0;
+	} else if (position == WACOM_TABLET_BUTTON_POS_BOTTOM) {
+		allocation->x -= requisition.width / 2.0;
+		allocation->y -= requisition.height;
+	} else if (position == WACOM_TABLET_BUTTON_POS_TOP) {
+		allocation->x -= requisition.width / 2.0;
+	}
+
+	return TRUE;
 }
 
 static void
@@ -1401,6 +1624,42 @@ gsd_wacom_osd_window_realized (GtkWidget *widget,
 	                         &osd_window->priv->tablet_area.height);
 	if (status == FALSE)
 		osd_window->priv->tablet_area = osd_window->priv->monitor_area;
+
+	/* Position the window at its expected postion before moving
+	 * to fullscreen, so the window will be on the right monitor.
+	 */
+	gtk_window_move (GTK_WINDOW (osd_window),
+	                 osd_window->priv->screen_area.x,
+	                 osd_window->priv->screen_area.y);
+
+	gtk_window_fullscreen (GTK_WINDOW (osd_window));
+	gtk_window_set_keep_above (GTK_WINDOW (osd_window), TRUE);
+}
+
+static gboolean
+gsd_wacom_osd_window_key_release_event (GtkWidget    *widget,
+					GdkEventKey  *event)
+{
+	GsdWacomOSDWindow *osd_window;
+	osd_window = GSD_WACOM_OSD_WINDOW (widget);
+
+	if (event->type != GDK_KEY_RELEASE)
+		goto out;
+
+	if (osd_window->priv->edition_mode) {
+		if (event->keyval == GDK_KEY_Escape && !gtk_widget_get_visible (osd_window->priv->editor))
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (osd_window->priv->change_mode_button),
+						      FALSE);
+
+		goto out;
+	}
+
+	gtk_widget_destroy (widget);
+
+ out:
+	GTK_WIDGET_CLASS (gsd_wacom_osd_window_parent_class)->key_release_event (widget, event);
+
+	return FALSE;
 }
 
 static void
@@ -1444,6 +1703,10 @@ gsd_wacom_osd_window_set_device (GsdWacomOSDWindow *osd_window,
 		gsd_wacom_osd_window_add_tablet_button (osd_window, tablet_button);
 	}
 	g_list_free (list);
+
+	g_clear_pointer (&osd_window->priv->regular_mode_message, g_free);
+	osd_window->priv->regular_mode_message = get_regular_mode_message (osd_window);
+
 }
 
 GsdWacomDevice *
@@ -1486,6 +1749,9 @@ gsd_wacom_osd_window_set_property (GObject        *object,
 	case PROP_OSD_WINDOW_MESSAGE:
 		gsd_wacom_osd_window_set_message (osd_window, g_value_get_string (value));
 		break;
+	case PROP_OSD_WINDOW_EDITION_MODE:
+		osd_window->priv->edition_mode = g_value_get_boolean (value);
+		break;
 	case PROP_OSD_WINDOW_GSD_WACOM_DEVICE:
 		gsd_wacom_osd_window_set_device (osd_window, g_value_get_object (value));
 		break;
@@ -1509,6 +1775,9 @@ gsd_wacom_osd_window_get_property (GObject        *object,
 	case PROP_OSD_WINDOW_MESSAGE:
 		g_value_set_string (value, osd_window->priv->message);
 		break;
+	case PROP_OSD_WINDOW_EDITION_MODE:
+		g_value_set_boolean (value, osd_window->priv->edition_mode);
+		break;
 	case PROP_OSD_WINDOW_GSD_WACOM_DEVICE:
 		g_value_set_object (value, (GObject*) osd_window->priv->pad);
 		break;
@@ -1524,19 +1793,45 @@ gsd_wacom_osd_window_set_active (GsdWacomOSDWindow    *osd_window,
 				 GtkDirectionType      dir,
 				 gboolean              active)
 {
+	GsdWacomOSDWindowPrivate *priv;
 	GList     *l;
 	gchar     *id;
 
 	g_return_if_fail (GSD_IS_WACOM_OSD_WINDOW (osd_window));
 	g_return_if_fail (button != NULL);
 
+	priv = osd_window->priv;
+
+	if (priv->current_button)
+		priv->current_button->priv->visible = TRUE;
+
 	id = get_tablet_button_id_name (button, dir);
-	for (l = osd_window->priv->buttons; l != NULL; l = l->next) {
+	for (l = priv->buttons; l != NULL; l = l->next) {
 		GsdWacomOSDButton *osd_button = l->data;
-		if (MATCH_ID (osd_button, id))
-			gsd_wacom_osd_button_set_active (osd_button, active);
+		if (MATCH_ID (osd_button, id)) {
+			if (priv->edition_mode && button->type != WACOM_TABLET_BUTTON_TYPE_HARDCODED)
+				priv->current_button = osd_button;
+			else
+				gsd_wacom_osd_button_set_active (osd_button, active);
+		}
 	}
 	g_free (id);
+
+	if (priv->edition_mode) {
+		if (priv->current_button)
+			priv->current_button->priv->visible = FALSE;
+
+		if (button->type == WACOM_TABLET_BUTTON_TYPE_HARDCODED)
+			return;
+
+		gtk_widget_hide (priv->editor);
+		gsd_wacom_button_editor_set_button (GSD_WACOM_BUTTON_EDITOR (priv->editor), button, dir);
+		gtk_widget_show (priv->editor);
+
+		redraw_window (osd_window);
+
+		return;
+	}
 }
 
 void
@@ -1565,8 +1860,27 @@ gsd_wacom_osd_window_set_mode (GsdWacomOSDWindow    *osd_window,
 			GsdWacomOSDButton *osd_button = l2->data;
 			gboolean           visible = (tablet_button->idx == mode - 1);
 
-			if (MATCH_ID (osd_button, id_up) || MATCH_ID (osd_button, id_down))
+			if (MATCH_ID (osd_button, id_up) || MATCH_ID (osd_button, id_down)) {
 				gsd_wacom_osd_button_set_visible (osd_button, visible);
+
+				if (osd_window->priv->current_button) {
+					gchar *current_id;
+					GtkDirectionType dir;
+
+					gsd_wacom_button_editor_get_button (GSD_WACOM_BUTTON_EDITOR (osd_window->priv->editor), &dir);
+					current_id = get_tablet_button_id_name (tablet_button, dir);
+
+					if (MATCH_ID (osd_button, current_id) && visible) {
+						osd_window->priv->current_button = osd_button;
+
+						gtk_widget_hide (osd_window->priv->editor);
+						gsd_wacom_button_editor_set_button (GSD_WACOM_BUTTON_EDITOR (osd_window->priv->editor), tablet_button, dir);
+						gtk_widget_show (osd_window->priv->editor);
+					}
+				}
+
+				redraw_window (osd_window);
+			}
 		}
 
 		g_free (id_up);
@@ -1576,6 +1890,23 @@ gsd_wacom_osd_window_set_mode (GsdWacomOSDWindow    *osd_window,
 	g_list_free (list);
 }
 
+gboolean
+gsd_wacom_osd_window_get_edition_mode (GsdWacomOSDWindow *osd_window)
+{
+	g_return_val_if_fail (GSD_IS_WACOM_OSD_WINDOW (osd_window), FALSE);
+
+	return osd_window->priv->edition_mode;
+}
+
+void
+gsd_wacom_osd_window_set_edition_mode (GsdWacomOSDWindow *osd_window, gboolean edition_mode)
+{
+	g_return_if_fail (GSD_IS_WACOM_OSD_WINDOW (osd_window));
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (osd_window->priv->change_mode_button),
+				      edition_mode);
+}
+
 GtkWidget *
 gsd_wacom_osd_window_new (GsdWacomDevice       *pad,
                           const gchar          *message)
@@ -1583,10 +1914,11 @@ gsd_wacom_osd_window_new (GsdWacomDevice       *pad,
 	GsdWacomOSDWindow *osd_window;
 	GdkScreen         *screen;
 	GdkVisual         *visual;
-	GtkWidget         *configure_button, *box;
+	GtkWidget         *button, *box, *overlay;
+	GtkStyleContext   *style_context;
 
 	osd_window = GSD_WACOM_OSD_WINDOW (g_object_new (GSD_TYPE_WACOM_OSD_WINDOW,
-	                                                 "type",              GTK_WINDOW_TOPLEVEL,
+	                                                 "type",              GTK_WINDOW_POPUP,
 	                                                 "skip-pager-hint",   TRUE,
 	                                                 "skip-taskbar-hint", TRUE,
 	                                                 "focus-on-map",      TRUE,
@@ -1605,24 +1937,51 @@ gsd_wacom_osd_window_new (GsdWacomDevice       *pad,
 		visual = gdk_screen_get_system_visual (screen);
 	gtk_widget_set_visual (GTK_WIDGET (osd_window), visual);
 
+	osd_window->priv->editor = gsd_wacom_button_editor_new ();
+	g_signal_connect (osd_window->priv->editor, "button-edited",
+			  G_CALLBACK (on_button_edited),
+			  osd_window);
+	g_signal_connect (osd_window->priv->editor, "done-editing",
+			  G_CALLBACK (on_button_editing_done),
+			  osd_window);
+
 	g_signal_connect (GTK_WIDGET (osd_window), "realize",
 	                  G_CALLBACK (gsd_wacom_osd_window_realized),
 	                  NULL);
-	g_signal_connect (GTK_WIDGET (osd_window), "map",
-	                  G_CALLBACK (gsd_wacom_osd_window_mapped),
-	                  NULL);
 
-	configure_button = create_osd_configure_button (osd_window);
+	overlay = gtk_overlay_new ();
+	gtk_container_add (GTK_CONTAINER (osd_window), overlay);
 
 	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_container_add (GTK_CONTAINER (osd_window), box);
-	gtk_box_pack_end (GTK_BOX (box), configure_button, FALSE, FALSE, 12);
-	gtk_widget_show_all(box);
+	gtk_container_add (GTK_CONTAINER (overlay), box);
 
-	g_signal_connect (configure_button, "clicked",
-			  G_CALLBACK (on_configure_button_clicked),
+	gtk_overlay_add_overlay (GTK_OVERLAY (overlay), osd_window->priv->editor);
+
+	button = gtk_toggle_button_new_with_label (_("Edit"));
+	g_object_set (button, "halign", GTK_ALIGN_CENTER, NULL);
+
+	style_context = gtk_widget_get_style_context (button);
+	gtk_style_context_add_class (style_context, "osd");
+
+	gtk_box_pack_end (GTK_BOX (box), button, FALSE, FALSE, 12);
+	osd_window->priv->change_mode_button = button;
+
+	gtk_widget_show (overlay);
+	gtk_widget_show (box);
+	gtk_widget_show (osd_window->priv->change_mode_button);
+
+	g_signal_connect (osd_window->priv->change_mode_button, "clicked",
+			  G_CALLBACK (on_change_mode_button_clicked),
 			  osd_window);
 
+	g_signal_connect (overlay, "get-child-position",
+			  G_CALLBACK (on_get_child_position),
+			  osd_window);
+
+	osd_window->priv->regular_mode_message = get_regular_mode_message (osd_window);
+
+	edition_mode_changed (osd_window);
+
 	return GTK_WIDGET (osd_window);
 }
 
@@ -1640,6 +1999,8 @@ gsd_wacom_osd_window_class_init (GsdWacomOSDWindowClass *klass)
 	gobject_class->finalize     = gsd_wacom_osd_window_finalize;
 	widget_class->draw          = gsd_wacom_osd_window_draw;
 	widget_class->motion_notify_event = gsd_wacom_osd_window_motion_notify_event;
+	widget_class->show          = gsd_wacom_osd_window_show;
+	widget_class->key_release_event = gsd_wacom_osd_window_key_release_event;
 
 	g_object_class_install_property (gobject_class,
 	                                 PROP_OSD_WINDOW_MESSAGE,
@@ -1656,15 +2017,32 @@ gsd_wacom_osd_window_class_init (GsdWacomOSDWindowClass *klass)
 	                                                      GSD_TYPE_WACOM_DEVICE,
 	                                                      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 
+	g_object_class_install_property (gobject_class,
+	                                 PROP_OSD_WINDOW_EDITION_MODE,
+	                                 g_param_spec_boolean ("edition-mode",
+							       "Edition mode",
+							       "The edition mode of the OSD Window.",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
+
 	g_type_class_add_private (klass, sizeof (GsdWacomOSDWindowPrivate));
 }
 
 static void
 gsd_wacom_osd_window_init (GsdWacomOSDWindow *osd_window)
 {
+	GtkSettings *settings;
 	osd_window->priv = GSD_WACOM_OSD_WINDOW_GET_PRIVATE (osd_window);
 	osd_window->priv->cursor_timeout = 0;
 	gtk_widget_add_events (GTK_WIDGET (osd_window), GDK_POINTER_MOTION_MASK);
+
+	osd_window->priv->edition_mode = FALSE;
+
+	osd_window->priv->edition_mode_message = get_edition_mode_message (osd_window);
+
+	settings = gtk_settings_get_default ();
+	g_object_set (G_OBJECT (settings), "gtk-application-prefer-dark-theme", TRUE, NULL);
 }
 
 static void
@@ -1683,6 +2061,8 @@ gsd_wacom_osd_window_finalize (GObject *object)
 	cursor_timeout_stop (osd_window);
 	g_clear_object (&priv->handle);
 	g_clear_pointer (&priv->message, g_free);
+	g_clear_pointer (&priv->regular_mode_message, g_free);
+	g_clear_pointer (&priv->edition_mode_message, g_free);
 	if (priv->buttons) {
 		g_list_free_full (priv->buttons, g_object_unref);
 		priv->buttons = NULL;
