@@ -170,6 +170,12 @@ class GSDTestCase(dbusmock.DBusTestCase):
                 ('Inhibit', 'ssss', 'h', 'ret = 5'),
             ], dbus_interface='org.freedesktop.DBus.Mock')
 
+        self.obj_logind.AddProperties('',
+                                      {
+                                          'CanSuspend': True,
+                                          'CanHibernate': False,
+                                      })
+
         # set log to nonblocking
         set_nonblock(self.logind.stdout)
 
@@ -179,6 +185,47 @@ class GSDTestCase(dbusmock.DBusTestCase):
         self.logind.terminate()
         self.logind.wait()
 
+    def start_mutter(self):
+        ''' start mock mutter DisplayConfig '''
+        self.mutter = self.spawn_server('org.gnome.Mutter.DisplayConfig',
+                                        '/org/gnome/Mutter/DisplayConfig',
+                                        'org.gnome.Mutter.DisplayConfig',
+                                        stdout=subprocess.PIPE)
+        self.obj_mutter = self.session_bus_con.get_object(
+            'org.gnome.Mutter.DisplayConfig', '/org/gnome/Mutter/DisplayConfig')
+
+        self.obj_mutter.AddMethods('',
+            [
+                ('GetResources', '', 'ua(uxiiiiiuaua{sv})a(uxiausauaua{sv})a(uxuud)ii', '''
+ret = (1, [(0, 1, 0, 0, 1024, 768, 0, 0, [0, 1, 2, 3, 4, 5, 6, 7], {}),
+           (1, 2, 0, 0, 0, 0, -1, 0, [], {})],
+          [(0, 3, 0, [0, 1], 'LVDS1', [0, 1], [], {'vendor': 'FOO',
+                                                   'product': '0x0000',
+                                                   'serial': '0x0000000',
+                                                   'display-name': 'Built-in Display',
+                                                   'backlight': -1,
+                                                   'primary': True })],
+          [(0, 5, 1024, 768, 60), (1, 6, 800, 600, 60)], 8192, 8192)'''),
+                ('ApplyConfiguration', 'uba(uiiiuaua{sv})a(ua{sv})', '', ''),
+                ('ChangeBacklight', 'uui', '', ''),
+                ('GetCrtcGamma', 'uu', 'aqaqaq', 'ret = ([],[],[])'),
+                ('SetCrtcGamma', 'uuaqaqaq', '', ''),
+            ], dbus_interface='org.freedesktop.DBus.Mock')
+
+        self.obj_mutter.AddProperties('',
+                                      {
+                                          'PowerSaveMode': 0,
+                                      })
+
+        # set log to nonblocking
+        set_nonblock(self.mutter.stdout)
+
+    def stop_mutter(self):
+        '''stop mock mutter'''
+
+        self.mutter.terminate()
+        self.mutter.wait()
+
     @classmethod
     def start_xorg(klass):
         '''start X.org server with dummy driver'''
